@@ -5,44 +5,41 @@ This module is internally used.
 
 import cmath
 import math
-from typing import Callable, Iterable, Iterator, List, NoReturn, Optional, Tuple, Union
+from typing import cast, Callable, Iterable, Iterator, List, NoReturn, Optional, Tuple, Type, TypeVar, Union
 
 import numpy as np
 
+from .typing import Targets
+
+_Op = TypeVar('_Op', bound='Operation')
+
 
 class Operation:
     """Abstract quantum circuit operation class."""
 
-    lowername: Optional[str] = None
+    lowername: str = ''
     """Lower name of the operation."""
     @property
     def uppername(self) -> str:
-        """Upper name of the gate."""
+        """Upper name of the operation."""
         return self.lowername.upper()
 
-    def __init__(self, targets, params=(), **kwargs) -> None:
-        if self.lowername is None:
+    def __init__(self, targets: Targets, params=()) -> None:
+        if self.lowername == '':
             raise ValueError(
                 f"{self.__class__.__name__}.lowername is not defined.")
         self.params = params
-        self.kwargs = kwargs
         self.targets = targets
 
     def target_iter(self, n_qubits: int) -> Iterator[int]:
         """The generator which yields the target qubits."""
         return slicing(self.targets, n_qubits)
 
-    def fallback(self, n_qubits: int) -> List['Operation']:
-        """Returns alternative operations to make equivalent circuit."""
-        raise NotImplementedError(
-            f"The fallback of {self.__class__.__name__} operation is not defined."
-        )
-
-    def dagger(self) -> 'Operation':
-        """Returns the Hermitian conjugate of `self`."""
-        raise NotImplementedError(
-            f"Hermitian conjugate of {self.__class__.__name__} operation is not provided."
-        )
+    @classmethod
+    def create(cls: Type[_Op], targets: Targets, params: tuple,
+               options: Optional[dict]) -> _Op:
+        """Create an operation."""
+        raise NotImplementedError(f"{cls.__name__}.create() is not defined.")
 
     def _str_args(self) -> str:
         """Returns printable string of args."""
@@ -72,6 +69,14 @@ class Operation:
         return f'{self.lowername}{str_args}{str_targets}'
 
 
+class IFallbackOperation(Operation):
+    """The interface of `fallback`"""
+    def fallback(self, n_qubits: int) -> List['Operation']:
+        """Get alternative operations"""
+        raise NotImplementedError(
+            f"fallback of {self.lowername} is not defined.")
+
+
 class Gate(Operation):
     """Abstract quantum gate class."""
     @property
@@ -79,15 +84,10 @@ class Gate(Operation):
         """Number of qubit arguments of this gate."""
         raise NotImplementedError()
 
-    def fallback(self, n_qubits: int) -> List['Gate']:
-        """Returns alternative gates to make equivalent circuit."""
-        raise NotImplementedError(
-            f"The fallback of {self.__class__.__name__} gate is not defined.")
-
     def dagger(self) -> 'Gate':
         """Returns the Hermitian conjugate of `self`."""
         raise NotImplementedError(
-            f"Hermitian conjugate of this gate is not provided.")
+            "Hermitian conjugate of this gate is not provided.")
 
     def matrix(self) -> np.ndarray:
         """Returns the matrix of implementations.
@@ -101,7 +101,7 @@ class Gate(Operation):
 class OneQubitGate(Gate):
     """Abstract quantum gate class for 1 qubit gate."""
 
-    u_params: Optional[Tuple[float, float, float, float]] = None
+    u_params: Optional[Tuple[float, float, float, float]]
     """Params for U gate."""
     @property
     def n_qargs(self) -> int:
@@ -115,20 +115,11 @@ class OneQubitGate(Gate):
             gates += fallback(t)
         return gates
 
-    def fallback(self, n_qubits: int) -> List['Gate']:
-        if self.u_params:
-            return [UGate(self.targets, *self.u_params, **self.kwargs)]
-        try:
-            mat = self.matrix()
-        except NotImplementedError:
-            return super().fallback(n_qubits)
-        return [Mat1Gate(self.targets, mat, **self.kwargs)]
-
 
 class TwoQubitGate(Gate):
     """Abstract quantum gate class for 2 qubits gate."""
 
-    cu_params: Optional[Tuple[float, float, float, float]] = None
+    cu_params: Optional[Tuple[float, float, float, float]]
 
     @property
     def n_qargs(self):
@@ -146,17 +137,21 @@ class TwoQubitGate(Gate):
             gates += fallback(c, t)
         return gates
 
-    def fallback(self, n_qubits: int) -> List['Gate']:
-        if self.cu_params:
-            return [CUGate(self.targets, *self.cu_params, **self.kwargs)]
-        return super().fallback(n_qubits)
-
 
 class HGate(OneQubitGate):
     """Hadamard gate"""
     lowername = "h"
     u_params = (-math.pi / 2.0, math.pi, 0.0, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'HGate':
+        if params or options:
+            raise ValueError(f"{cls.__name__} doesn't take parameters")
+        return cls(targets)
+
     def dagger(self):
         return self
 
@@ -164,19 +159,28 @@ class HGate(OneQubitGate):
         return np.array([[1, 1], [1, -1]], dtype=complex) / math.sqrt(2)
 
 
-class IGate(OneQubitGate):
+class IGate(OneQubitGate, IFallbackOperation):
     """Identity gate"""
     lowername = "i"
     u_params = (0.0, 0.0, 0.0, 0.0)
 
-    def fallback(self, n_qubits):
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'IGate':
+        if params or options:
+            raise ValueError(f"{cls.__name__} doesn't take parameters")
+        return cls(targets)
+
+    def fallback(self, _):
         return []
 
     def dagger(self):
         return self
 
     def matrix(self):
-        return np.eye(2)
+        return np.eye(2, dtype=complex)
 
 
 class Mat1Gate(OneQubitGate):
@@ -188,20 +192,25 @@ class Mat1Gate(OneQubitGate):
     lowername = "mat1"
     u_params = None
 
-    def __init__(self, targets, mat: np.ndarray, **kwargs):
-        super().__init__(targets, (mat, ), **kwargs)
+    def __init__(self, targets, mat: np.ndarray):
+        super().__init__(targets, (mat, ))
         self.mat = mat
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'Mat1Gate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
     def dagger(self):
-        return Mat1Gate(self.targets, self.mat.T.conjugate(), **self.kwargs)
+        return Mat1Gate(self.targets, self.mat.T.conjugate())
 
     def matrix(self):
         return self.mat
 
-    def fallback(self, n_qubits: int) -> List['Gate']:
-        raise NotImplementedError(
-            'Fallback implementation for Mat1Gate is not available.')
-
 
 class PhaseGate(OneQubitGate):
     """Rotate-Z gate but global phase is different.
@@ -215,29 +224,52 @@ class PhaseGate(OneQubitGate):
     But matrix of phase gate is simpler than RZ gate or U1 gate.)"""
     lowername = "phase"
 
-    def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta, ))
         self.theta = theta
         self.u_params = (0.0, self.theta, 0.0, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'PhaseGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
     def dagger(self):
-        return PhaseGate(self.targets, -self.theta, **self.kwargs)
+        return PhaseGate(self.targets, -self.theta)
 
     def matrix(self):
-        return np.array([[1, 0], [0, cmath.exp(1j * self.theta)]], dtype=complex)
+        return np.array([[1, 0], [0, cmath.exp(1j * self.theta)]],
+                        dtype=complex)
 
 
-class RXGate(OneQubitGate):
+class RXGate(OneQubitGate, IFallbackOperation):
     """Rotate-X gate"""
     lowername = "rx"
 
-    def __init__(self, targets, theta: float, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta: float):
+        super().__init__(targets, (theta, ))
         self.theta = theta
         self.u_params = (theta, -math.pi / 2.0, math.pi / 2.0, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'RXGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
     def dagger(self):
-        return RXGate(self.targets, -self.theta, **self.kwargs)
+        return RXGate(self.targets, -self.theta)
+
+    def fallback(self, _):
+        assert self.u_params is not None
+        return [UGate.create(self.targets, self.u_params)]
 
     def matrix(self):
         t = self.theta * 0.5
@@ -246,17 +278,30 @@ class RXGate(OneQubitGate):
         return np.array([[a, b], [b, a]], dtype=complex)
 
 
-class RYGate(OneQubitGate):
+class RYGate(OneQubitGate, IFallbackOperation):
     """Rotate-Y gate"""
     lowername = "ry"
 
-    def __init__(self, targets, theta: float, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta: float):
+        super().__init__(targets, (theta, ))
         self.theta = theta
         self.u_params = (theta, 0.0, 0.0, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'RYGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
     def dagger(self):
-        return RYGate(self.targets, -self.theta, **self.kwargs)
+        return RYGate(self.targets, -self.theta)
+
+    def fallback(self, _):
+        assert self.u_params is not None
+        return [UGate.create(self.targets, self.u_params)]
 
     def matrix(self):
         t = self.theta * 0.5
@@ -265,30 +310,52 @@ class RYGate(OneQubitGate):
         return np.array([[a, -b], [b, a]], dtype=complex)
 
 
-class RZGate(OneQubitGate):
+class RZGate(OneQubitGate, IFallbackOperation):
     """Rotate-Z gate"""
     lowername = "rz"
 
-    def __init__(self, targets, theta: float, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta: float):
+        super().__init__(targets, (theta, ))
         self.theta = theta
         self.u_params = (0.0, 0.0, theta, -0.5 * theta)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'RZGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
     def dagger(self):
-        return RZGate(self.targets, -self.theta, **self.kwargs)
+        return RZGate(self.targets, -self.theta)
+
+    def fallback(self, _):
+        assert self.u_params is not None
+        return [UGate.create(self.targets, self.u_params)]
 
     def matrix(self):
         a = cmath.exp(0.5j * self.theta)
         return np.array([[a.conjugate(), 0], [0, a]], dtype=complex)
 
 
-class SGate(OneQubitGate):
+class SGate(OneQubitGate, IFallbackOperation):
     """S gate"""
     lowername = "s"
     u_params = (0.0, 0.0, math.pi / 2, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'SGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
-        return SDagGate(self.targets, self.params, **self.kwargs)
+        return SDagGate(self.targets, self.params)
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_target_iter(
@@ -298,13 +365,22 @@ class SGate(OneQubitGate):
         return np.array([[1, 0], [0, 1j]])
 
 
-class SDagGate(OneQubitGate):
+class SDagGate(OneQubitGate, IFallbackOperation):
     """Dagger of S gate"""
     lowername = "sdg"
     u_params = (0.0, 0.0, -math.pi / 2, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'SDagGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
-        return SGate(self.targets, self.params, **self.kwargs)
+        return SGate(self.targets, self.params)
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_target_iter(
@@ -314,63 +390,107 @@ class SDagGate(OneQubitGate):
         return np.array([[1, 0], [0, -1j]])
 
 
-class SXGate(OneQubitGate):
+class SXGate(OneQubitGate, IFallbackOperation):
     """sqrt(X) gate
 
     This is equivalent as RX(π/2) * (1 + i) / √2."""
     lowername = "sx"
     u_params = (math.pi / 2.0, -math.pi / 2.0, math.pi / 2.0, math.pi / 4.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'SXGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
-        return SXDagGate(self.targets, self.params, **self.kwargs)
+        return SXDagGate(self.targets, self.params)
+
+    def fallback(self, _):
+        assert self.u_params is not None
+        return [UGate.create(self.targets, self.u_params)]
 
     def matrix(self):
         return np.array([[1 + 1j, 1 - 1j], [1 - 1j, 1 + 1j]])
 
 
-class SXDagGate(OneQubitGate):
+class SXDagGate(OneQubitGate, IFallbackOperation):
     """sqrt(X)† gate"""
     lowername = "sxdg"
     u_params = (-math.pi / 2.0, -math.pi / 2.0, math.pi / 2.0, -math.pi / 4.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'SXDagGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
-        return SXGate(self.targets, self.params, **self.kwargs)
+        return SXGate(self.targets, self.params)
+
+    def fallback(self, _):
+        assert self.u_params is not None
+        return [UGate.create(self.targets, self.u_params)]
 
     def matrix(self):
         return np.array([[1 - 1j, 1 + 1j], [1 + 1j, 1 - 1j]])
 
 
-class TGate(OneQubitGate):
+class TGate(OneQubitGate, IFallbackOperation):
     """T ($\\pi/8$) gate"""
     lowername = "t"
     u_params = (0, 0, math.pi / 4.0, 0.0)
 
-    def dagger(self):
-        return TDagGate(self.targets, self.params, **self.kwargs)
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'TGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
 
-    def fallback(self, n_qubits):
-        return [PhaseGate(self.targets, math.pi / 4, **self.kwargs)]
+    def dagger(self):
+        return TDagGate(self.targets, self.params)
+
+    def fallback(self, _):
+        return [PhaseGate(self.targets, math.pi / 4)]
 
     def matrix(self):
-        return np.array([[1, 0], [0, math.exp(math.pi * 0.25)]])
+        return np.array([[1, 0], [0, cmath.exp(math.pi * 0.25j)]])
 
 
-class TDagGate(OneQubitGate):
+class TDagGate(OneQubitGate, IFallbackOperation):
     """Dagger of T ($\\pi/8$) gate"""
     lowername = "tdg"
     u_params = (0, 0, -math.pi / 4.0, 0.0)
 
-    def dagger(self):
-        return TGate(self.targets, self.params, **self.kwargs)
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'TDagGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
 
-    def fallback(self, n_qubits):
-        return [PhaseGate(self.targets, -math.pi / 4, **self.kwargs)]
+    def dagger(self):
+        return TGate(self.targets, self.params)
+
+    def fallback(self, _):
+        return [PhaseGate(self.targets, -math.pi / 4)]
 
     def matrix(self):
-        return np.array([[1, 0], [0, math.exp(math.pi * -0.25)]])
+        return np.array([[1, 0], [0, cmath.exp(math.pi * -0.25j)]])
 
 
-class ToffoliGate(Gate):
+class ToffoliGate(Gate, IFallbackOperation):
     """Toffoli (CCX) gate"""
     lowername = "ccx"
 
@@ -378,6 +498,15 @@ class ToffoliGate(Gate):
     def n_qargs(self):
         return 3
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'ToffoliGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
@@ -413,22 +542,26 @@ class UGate(OneQubitGate):
                  theta: float,
                  phi: float,
                  lam: float,
-                 gamma: float = 0.0,
-                 **kwargs):
-        super().__init__(targets, (theta, phi, lam, gamma), **kwargs)
+                 gamma: float = 0.0):
+        super().__init__(targets, (theta, phi, lam, gamma))
         self.theta = theta
         self.phi = phi
         self.lam = lam
         self.gamma = gamma
         self.u_params = (theta, phi, lam, gamma)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'UGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, *params)
+
     def dagger(self):
         return UGate(self.targets, -self.theta, -self.lam, -self.phi,
-                     -self.gamma, **self.kwargs)
-
-    def fallback(self, n_qubits: int) -> List['Gate']:
-        raise NotImplementedError(
-            'Fallback implementation for UGate is not available.')
+                     -self.gamma)
 
     def matrix(self):
         t, p, l, g = self.params
@@ -447,6 +580,15 @@ class XGate(OneQubitGate):
     lowername = "x"
     u_params = (math.pi, 0.0, math.pi, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'XGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
@@ -459,6 +601,15 @@ class YGate(OneQubitGate):
     lowername = "y"
     u_params = (math.pi, math.pi / 2, math.pi / 2, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'YGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
@@ -471,6 +622,15 @@ class ZGate(OneQubitGate):
     lowername = "z"
     u_params = (0.0, 0.0, math.pi, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'ZGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
@@ -478,7 +638,7 @@ class ZGate(OneQubitGate):
         return np.array([[1, 0], [0, -1]], dtype=complex)
 
 
-class CCZGate(Gate):
+class CCZGate(Gate, IFallbackOperation):
     """2-Controlled Z gate"""
     lowername = "ccz"
 
@@ -486,6 +646,15 @@ class CCZGate(Gate):
     def n_qargs(self):
         return 3
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CCZGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def fallback(self, n_qubits):
         c1, c2, t = self.targets
         return [
@@ -515,14 +684,27 @@ class CCZGate(Gate):
                         dtype=complex)
 
 
-class CHGate(TwoQubitGate):
+class CHGate(TwoQubitGate, IFallbackOperation):
     """Controlled-H gate"""
     lowername = "ch"
     cu_params = (-math.pi / 2.0, math.pi, 0.0, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CHGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
+    def fallback(self, _):
+        assert self.cu_params is not None
+        return [CUGate.create(self.targets, self.cu_params)]
+
     def matrix(self):
         a = 1.0 / math.sqrt(2)
         return np.array(
@@ -530,17 +712,30 @@ class CHGate(TwoQubitGate):
             dtype=complex)
 
 
-class CPhaseGate(TwoQubitGate):
+class CPhaseGate(TwoQubitGate, IFallbackOperation):
     """Rotate-Z gate but phase is different."""
     lowername = "cphase"
 
-    def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta, ))
         self.theta = theta
         self.cu_params = (0.0, self.theta, 0.0, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CPhaseGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
+    def fallback(self, _):
+        assert self.cu_params is not None
+        return [CUGate.create(self.targets, self.cu_params)]
+
     def dagger(self):
-        return CPhaseGate(self.targets, -self.theta, **self.kwargs)
+        return CPhaseGate(self.targets, -self.theta)
 
     def matrix(self):
         return np.array([[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0],
@@ -548,17 +743,30 @@ class CPhaseGate(TwoQubitGate):
                         dtype=complex)
 
 
-class CRXGate(TwoQubitGate):
+class CRXGate(TwoQubitGate, IFallbackOperation):
     """Rotate-X gate"""
     lowername = "crx"
 
-    def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta, ))
         self.theta = theta
         self.cu_params = (theta, -math.pi / 2.0, math.pi / 2.0, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CRXGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
     def dagger(self):
-        return CRXGate(self.targets, -self.theta, **self.kwargs)
+        return CRXGate(self.targets, -self.theta)
+
+    def fallback(self, _):
+        assert self.cu_params is not None
+        return [CUGate.create(self.targets, self.cu_params)]
 
     def matrix(self):
         t = self.theta * 0.5
@@ -569,17 +777,30 @@ class CRXGate(TwoQubitGate):
             dtype=complex)
 
 
-class CRYGate(TwoQubitGate):
+class CRYGate(TwoQubitGate, IFallbackOperation):
     """Rotate-Y gate"""
     lowername = "cry"
 
-    def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta, ))
         self.theta = theta
         self.cu_params = (theta, 0.0, 0.0, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CRYGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
     def dagger(self):
-        return CRYGate(self.targets, -self.theta, **self.kwargs)
+        return CRYGate(self.targets, -self.theta)
+
+    def fallback(self, _):
+        assert self.cu_params is not None
+        return [CUGate.create(self.targets, self.cu_params)]
 
     def matrix(self):
         t = self.theta * 0.5
@@ -590,17 +811,30 @@ class CRYGate(TwoQubitGate):
             dtype=complex)
 
 
-class CRZGate(TwoQubitGate):
+class CRZGate(TwoQubitGate, IFallbackOperation):
     """Rotate-Z gate"""
     lowername = "crz"
 
-    def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta, ))
         self.theta = theta
         self.cu_params = (0.0, 0.0, theta, -0.5 * theta)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CRZGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params[0])
+
+    def fallback(self, _):
+        assert self.cu_params is not None
+        return [CUGate.create(self.targets, self.cu_params)]
+
     def dagger(self):
-        return CRZGate(self.targets, -self.theta, **self.kwargs)
+        return CRZGate(self.targets, -self.theta)
 
     def matrix(self):
         a = cmath.exp(0.5j * self.theta)
@@ -609,7 +843,7 @@ class CRZGate(TwoQubitGate):
                         dtype=complex)
 
 
-class CSwapGate(Gate):
+class CSwapGate(Gate, IFallbackOperation):
     """Controlled SWAP gate"""
     lowername = "cswap"
 
@@ -617,6 +851,15 @@ class CSwapGate(Gate):
     def n_qargs(self):
         return 3
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CSwapGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
@@ -632,7 +875,7 @@ class CSwapGate(Gate):
                         dtype=complex)
 
 
-class CUGate(TwoQubitGate):
+class CUGate(TwoQubitGate, IFallbackOperation):
     """Controlled-U gate."""
     lowername = "cu"
 
@@ -641,28 +884,36 @@ class CUGate(TwoQubitGate):
                  theta: float,
                  phi: float,
                  lam: float,
-                 gamma: float = 0.0,
-                 **kwargs):
-        super().__init__(targets, (theta, phi, lam, gamma), **kwargs)
+                 gamma: float = 0.0):
+        super().__init__(targets, (theta, phi, lam, gamma))
         self.theta = theta
         self.phi = phi
         self.lam = lam
         self.gamma = gamma
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CUGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, *params)
+
     def dagger(self):
         return CUGate(self.targets, -self.theta, -self.lam, -self.phi,
-                      -self.gamma, **self.kwargs)
+                      -self.gamma)
 
     def fallback(self, n_qubits: int) -> List['Gate']:
         theta, phi, lam, gamma = self.params
         return self._make_fallback_for_control_target_iter(
             n_qubits, lambda c, t: [
-                PhaseGate((c,), gamma + 0.5 * (lam + phi)),
-                PhaseGate((t,), 0.5 * (lam - phi)),
+                PhaseGate((c, ), gamma + 0.5 * (lam + phi)),
+                PhaseGate((t, ), 0.5 * (lam - phi)),
                 CXGate((c, t)),
-                UGate((t,), -0.5 * theta, 0.0, -0.5 * (phi + lam)),
+                UGate((t, ), -0.5 * theta, 0.0, -0.5 * (phi + lam)),
                 CXGate((c, t)),
-                UGate((t,), 0.5 * theta, phi, 0.0)
+                UGate((t, ), 0.5 * theta, phi, 0.0)
             ])
 
     def matrix(self):
@@ -687,6 +938,15 @@ class CXGate(TwoQubitGate):
     lowername = "cx"
     cu_params = (math.pi, 0.0, math.pi, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CXGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
@@ -696,14 +956,27 @@ class CXGate(TwoQubitGate):
             dtype=complex)
 
 
-class CYGate(TwoQubitGate):
+class CYGate(TwoQubitGate, IFallbackOperation):
     """Controlled-Y gate"""
     lowername = "cy"
     cu_params = (math.pi, math.pi / 2, math.pi / 2, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CYGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
+    def fallback(self, _):
+        assert self.cu_params is not None
+        return [CUGate.create(self.targets, self.cu_params)]
+
     def matrix(self):
         return np.array(
             [[1, 0, 0, 0], [0, 0, -1j, 0], [0, 0, 1, 0], [0, 1j, 0, 0]],
@@ -715,6 +988,15 @@ class CZGate(TwoQubitGate):
     lowername = "cz"
     cu_params = (0.0, 0.0, math.pi, 0.0)
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'CZGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def dagger(self):
         return self
 
@@ -724,16 +1006,25 @@ class CZGate(TwoQubitGate):
             dtype=complex)
 
 
-class RXXGate(TwoQubitGate):
+class RXXGate(TwoQubitGate, IFallbackOperation):
     """Rotate-XX gate"""
     lowername = "rxx"
 
-    def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta, ))
         self.theta = theta
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'RXXGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, *params)
+
     def dagger(self):
-        return RXXGate(self.targets, -self.theta, **self.kwargs)
+        return RXXGate(self.targets, -self.theta)
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
@@ -753,16 +1044,25 @@ class RXXGate(TwoQubitGate):
             dtype=complex)
 
 
-class RYYGate(TwoQubitGate):
+class RYYGate(TwoQubitGate, IFallbackOperation):
     """Rotate-YY gate"""
     lowername = "ryy"
 
-    def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta, ))
         self.theta = theta
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'RYYGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, *params)
+
     def dagger(self):
-        return RYYGate(self.targets, -self.theta, **self.kwargs)
+        return RYYGate(self.targets, -self.theta)
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
@@ -782,16 +1082,25 @@ class RYYGate(TwoQubitGate):
             dtype=complex)
 
 
-class RZZGate(TwoQubitGate):
+class RZZGate(TwoQubitGate, IFallbackOperation):
     """Rotate-ZZ gate"""
     lowername = "rzz"
 
-    def __init__(self, targets, theta, **kwargs):
-        super().__init__(targets, (theta, ), **kwargs)
+    def __init__(self, targets, theta):
+        super().__init__(targets, (theta, ))
         self.theta = theta
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'RZZGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, *params)
+
     def dagger(self):
-        return RZZGate(self.targets, -self.theta, **self.kwargs)
+        return RZZGate(self.targets, -self.theta)
 
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
@@ -807,13 +1116,22 @@ class RZZGate(TwoQubitGate):
                         dtype=complex)
 
 
-class SwapGate(TwoQubitGate):
+class SwapGate(TwoQubitGate, IFallbackOperation):
     """Swap gate"""
     lowername = "swap"
 
     def dagger(self):
         return self
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'SwapGate':
+        if options:
+            raise ValueError(f"{cls.__name__} doesn't take options")
+        return cls(targets, params)
+
     def fallback(self, n_qubits):
         return self._make_fallback_for_control_target_iter(
             n_qubits,
@@ -826,7 +1144,7 @@ class SwapGate(TwoQubitGate):
                          [0, 0, 0, 1]])
 
 
-class ZZGate(TwoQubitGate):
+class ZZGate(TwoQubitGate, IFallbackOperation):
     """ZZ gate
 
     This gate is a basis two-qubit gate for some kinds of trapped-ion based machines.
@@ -834,8 +1152,17 @@ class ZZGate(TwoQubitGate):
     """
     lowername = "zz"
 
-    def __init__(self, targets, **kwargs):
-        super().__init__(targets, (), **kwargs)
+    def __init__(self, targets):
+        super().__init__(targets, ())
+
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'ZZGate':
+        if params or options:
+            raise ValueError(f"{cls.__name__} doesn't take params")
+        return cls(targets)
 
     def dagger(self):
         return self
@@ -857,6 +1184,28 @@ class Measurement(Operation):
     """Measurement operation"""
     lowername = "measure"
 
+    def __init__(self, targets: Targets, options: Optional[dict]):
+        super().__init__(targets, ())
+        if options is None:
+            options = {}
+        key = options.get("key")
+        if key is not None:
+            key = str(key)
+        duplicated = options.get("duplicated")
+        if duplicated is not None:
+            duplicated = str(duplicated)
+        self.key = key
+        self.duplicated = duplicated
+
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'Measurement':
+        if params:
+            raise ValueError(f"{cls.__name__} doesn't take params")
+        return cls(targets, options)
+
     # Ad-hoc copy and paste programming.
     def target_iter(self, n_qubits):
         """The generator which yields the target qubits."""
@@ -867,6 +1216,15 @@ class Reset(Operation):
     """Reset operation"""
     lowername = "reset"
 
+    @classmethod
+    def create(cls,
+               targets: Targets,
+               params: tuple,
+               options: Optional[dict] = None) -> 'Reset':
+        if params or options:
+            raise ValueError(f"{cls.__name__} doesn't take params")
+        return cls(targets, params)
+
     # Ad-hoc copy and paste programming.
     def target_iter(self, n_qubits):
         """The generator which yields the target qubits."""
@@ -908,7 +1266,7 @@ def slicing_singlevalue(arg: Union[slice, int], length: int) -> Iterator[int]:
         yield i
 
 
-def slicing(args: Tuple[Union[slice, int], ...], length: int) -> Iterator[int]:
+def slicing(args: Targets, length: int) -> Iterator[int]:
     """Internally used."""
     if isinstance(args, tuple):
         for arg in args:
@@ -917,8 +1275,7 @@ def slicing(args: Tuple[Union[slice, int], ...], length: int) -> Iterator[int]:
         yield from slicing_singlevalue(args, length)
 
 
-def qubit_pairs(args: Tuple[Tuple[Union[slice, int], ...],
-                            Tuple[Union[slice, int], ...]],
+def qubit_pairs(args: Tuple[Targets, Targets],
                 length: int) -> Iterator[Tuple[int, int]]:
     """Internally used."""
     if not isinstance(args, tuple):
@@ -937,7 +1294,7 @@ def qubit_pairs(args: Tuple[Tuple[Union[slice, int], ...],
     return zip(controls, targets)
 
 
-def get_maximum_index(indices: Union[Tuple[int, ...], int]) -> int:
+def get_maximum_index(indices: Targets) -> int:
     """Internally used."""
     def _maximum_idx_single(idx: int):
         if isinstance(idx, slice):
